function EstimationModel(Freq,Type,Code,NumSpec,SetInst)
%This function compute the OLS regression and it applies the trimmed approach.

%INPUT:
%Freq= 1: monthly; 2: quarterly;
%Type= 1: financial models; 2: macroeconomic models;
%Code= 10: time-invariant specifications; 11: time-varying specification
%NumSpec=specification number: 
%for financial models:NumSpec= 1 CAPM, 2 FF, 3 CAR, 4 5FF, 5 HXZ, 6 FF and QMJ, 7 FF and BAB (see Table 1);
%for macroeconomic models: NumSpec= 1 CCAPM, 2 EZ, 3 NDC and DC, 4 YO, 5 LVX (see Table 2)
%SetInst= seto of isntruments involved in the conditional specification: 1 Z_t-1={1,divY_t-1}, 2 Z_t-1={1,divY_t-1} and Z_{i,t-1}=bm_{i,t-1}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load Datasets
% The dataset contains the following data:
% R=returns, I=indicator monthly data r for unbalanced characteristic, BM=book-to-mkt
% Ti= time-series observations for each asset, T=TS size, n=CS size
if Freq==1;		%monthly data
    load Wspace_CRSPCMST_ret R I BM Ti T n; %Dataset after matching CRSP and Compustat Jan68-Dec11
    load Wspace_FactorsM Rf 				%Risk free rate
else			%quarterly data
    load Wspace_CRSPCMST_retQ  R I BM Ti T n;  %Dataset after matching CRSP and Compustat Jan68-Dec11
    load Wspace_FactorsQ Rf 				%Risk free rate
end

% Excess Returns
ExcR=(I==1).*(R-repmat(Rf,1,n))+0;       %Txn matrix of excess return

% K= number of factors (exclude the constant)
[F,K]=DefSpecification(NumSpec,Type,Freq);
F=F.*10;                                 %Rescaled factors

if Code==10;    %Time-invariant specifications
    %Estimation of no-time varying coefficients and residuals
    %Time-series regression
    [a,b,res,CN,~,~,tau,R2,R2adj,SYSRisk,IDIRisk]=TSRegress_CN_V(ExcR,F,I,Ti,n,T,K);
    %Trimming Approach
    chi1=15;
    if Freq==1;
        chi2=T/60;
    else
        chi2=T/20;
    end;
    ind1=find(CN<chi1);
    I=I(:,ind1); res=res(:,ind1); tau=tau(ind1,:); Ti=Ti(ind1,:); ExcR=ExcR(:,ind1);
    a=a(ind1,:); b=b(ind1,:);
    R2=R2(ind1,:); R2adj=R2adj(ind1,:); SYSRisk=SYSRisk(ind1,:); IDIRisk=IDIRisk(ind1,:);
    ind2=(tau<=chi2);
    I=I(:,ind2); res=res(:,ind2); tau=tau(ind2,:); Ti=Ti(ind2,:); ExcR=ExcR(:,ind2);
    a=a(ind2,:); b=b(ind2,:);
    R2=R2(ind2,:); R2adj=R2adj(ind2,:); SYSRisk=SYSRisk(ind2,:); IDIRisk=IDIRisk(ind2,:);
    nchi=size(tau,1);    %trimmed cross-sectional dimension
    
else			%Time-varying specifications

    %instrumental definition
    [Z,Zi,d1,d2,AssBeta]=InstrumentalChoice(T,n,K,Ti,BM,I,Freq,SetInst);
    %Estimation of time varying coefficients and residuals
    % In the time-varying model we loose one observation
    ExcR=ExcR(2:T,:); I=I(2:T,:); Ti=sum(I)';
    d=d1+d2;
    %First pass (see Section 3.2 GOS(2016))
    %Estimation of betas
    [beta1,~,res,CN,~,~,R2,R2adj,SYSRisk,IDIRisk]=TSRegress_CN_V_cond(ExcR,F,Z,Zi,I,Ti,n,T,d1,d2,d,AssBeta);
    %Defition of tau
    tau=(T-1)./Ti;      %nx1
    % Trimming Approach
    chi1=15;
    if Freq==1;
        chi2=(T-1)/60;
    else
        chi2=(T-1)/20;
    end
    ind1=find(CN<=chi1);
    I=I(:,ind1); res=res(:,ind1); tau=tau(ind1,:); Ti=Ti(ind1,:); ExcR=ExcR(:,ind1);
    beta1=beta1(:,ind1); R2=R2(ind1,:); R2adj=R2adj(ind1,:); SYSRisk=SYSRisk(ind1,:); IDIRisk=IDIRisk(ind1,:);
    ind2=find(tau<=chi2); 
    I=I(:,ind2); res=res(:,ind2); tau=tau(ind2,:); Ti=Ti(ind2,:); ExcR=ExcR(:,ind2);
    beta1=beta1(:,ind2); R2=R2(ind2,:); R2adj=R2adj(ind2,:); SYSRisk=SYSRisk(ind2,:); IDIRisk=IDIRisk(ind2,:);
    nchi=size(tau,1);   %trimmed cross-sectional dimension
end;

%Save results
clearvars chi1 chi2 CN ind1 ind2 tau BM R 
if Code==10 %Time invariant models
    fname=sprintf('ESTRES_Freq%d_Type%d_Code%d_Spec%d',Freq,Type,Code,NumSpec);    
else
    fname=sprintf('ESTRES_Freq%d_Type%d_Code%d_Spec%d_SetInst%d',Freq,Type,Code,NumSpec,SetInst);    
end
save(fname);
